//////////////////////////////////////////////////////////
//Bildverschlüsselung durch Permutation als Beispiel 
//für das Laden und Speichern großer Textdateien.
///////////////////////////////////////////////////////

PImage bild, bildVerschluesselt;
String[] permSchluesselX, permSchluesselY ; 
int n=0;
int dimX=0; //Bildbreite
int dimY=0; //Bildhöhe
boolean fertig = false;
PFont font;

////////////////////////////////////////////////////  
//Hat das Bild die Dimension (dimX,dimY), dann sollte
//die Anzahl der Permutationen n mindestens n = dimX*dimY*4
//sein.Beide, permSchluesselX und permSchluesselY haben 
//n Elemente. Wobei in permSchluesselX nur Zahlen zwischen
//0 und dimX-1, während in permSchluesselY nur Zahlen
//zwischen 0 und dimY-1 vorkommen dürfen.
//Immer zwei Zahlen aus permSchluesselX und zwei Zahlen
//aus permSchluesselY bestimmen einen Farbentausch.
//Beispiel: 
//permSchluesselX beginne mit den Werten 34,116
//permSchluesselY beginne mit den Werten 150,82
//Dann tauschen die Pixel an (34,150) die Farbe 
//die Farbe mit (116,82).
//Beim Entschluesseln wird einfach wieder in umgekehrter
//Reihenfolge zurückgetauscht.
////////////////////////////////////////////////////
//Erstes Vertauschen wie oben beschrieben
//////////////////////////////////////////////////
void verschluesseln(){ 
 int ax, ay, bx, by;
 permSchluesselX = loadStrings("permSchluesselX.txt");
 permSchluesselY = loadStrings("permSchluesselY.txt");

 color tauschA = color(0,0,0);
 color tauschB = color(0,0,0);
 PImage tmpBild = createImage(dimX,dimY,RGB);
 for (int i = 0; i < dimX; i++) {
    for (int j = 0; j < dimY; j++) {
      tmpBild.set(i,j,bild.get(i,j));
    }
  }
 for(int i = 0; i<n; i= i+2){
    ax = Integer.parseInt(permSchluesselX[i]);
    ay = Integer.parseInt(permSchluesselY[i]);
    bx = Integer.parseInt(permSchluesselX[i+1]);
    by = Integer.parseInt(permSchluesselY[i+1]);
    tauschA = tmpBild.get(ax,ay);
    tauschB = tmpBild.get(bx,by); 
    tmpBild.set(ax,ay,tauschB) ;
    tmpBild.set(bx,by,tauschA) ;
  }
  fertig = true;
  bild = tmpBild; 
  bild.save("data/bildVerschluesselt.png");
}
//////////////////////////////////////////////////
//Zurücktauschen in umgekehrter Reihenfolge
////////////////////////////////////////////////
void entschluesseln(){
 int ax, ay, bx, by;
 permSchluesselX = loadStrings("permSchluesselX.txt");
 permSchluesselY = loadStrings("permSchluesselY.txt");
 bildVerschluesselt = loadImage("bildVerschluesselt.png");
 color tauschA = color(0,0,0);
 color tauschB = color(0,0,0);
 PImage tmpBild = createImage(dimX,dimY,RGB);
 for (int i = 0; i < dimX; i++) {
    for (int j = 0; j < dimY; j++) {
      tmpBild.set(i,j,bildVerschluesselt.get(i,j));
    }
  }
 for(int i = 0; i<n; i= i+2){
    ax = Integer.parseInt(permSchluesselX[n-1-i]);
    ay = Integer.parseInt(permSchluesselY[n-1-i]);
    bx = Integer.parseInt(permSchluesselX[n-2-i]);
    by = Integer.parseInt(permSchluesselY[n-2-i]);
    tauschA = tmpBild.get(ax,ay);
    tauschB = tmpBild.get(bx,by);
    tmpBild.set(ax,ay,tauschB) ;
    tmpBild.set(bx,by,tauschA) ;
  }
  fertig = true;
  bild = tmpBild; 
  bild.save("bildEntschlusselt.png");
}
//////////////////////////////////////////////////
//n Pseudozufallszahlen bestimmen und speichern
//////////////////////////////////////////////////
void permutationsSchluesselBestimmen(int n){
 
  permSchluesselX = new String[n];
  for (int i = 0; i < n; i ++ ) {
    permSchluesselX[i] = str(int(random(dimX)));    
  }
  saveStrings("data/permSchluesselX.txt", permSchluesselX);

  permSchluesselY = new String[n];
  for (int i = 0; i < n; i ++ ) {
    permSchluesselY[i] = str(int(random(dimY)));    
  }
  saveStrings("data/permSchluesselY.txt", permSchluesselY);
 
}
//////////////////////////////////////////////////  
//Optionen des Users
/////////////////////////////////////////////////////
void keyPressed() { 
  if (key == 's' ) {
    permutationsSchluesselBestimmen(n);
  } 
 if (key == 'v' ) {
    verschluesseln();
  } 
  if (key == 'e' ) {
    entschluesseln();
  } 
}

///////////////////////////////////////////////

///////////////////////////////////////////////////
void setup() {
 File fBild = new File(dataPath("bild.png"));
 File fVerschl = new File(dataPath("bildVerschluesselt.png"));
 if (fBild.exists())
 {
  bild = loadImage("bild.png");
 }
 if ( fVerschl.exists())
 {
  bildVerschluesselt = loadImage("bildVerschluesselt.png");
 }

 dimX = bild.width; 
 dimY = bild.height;
 n = dimX*dimY*4;// Kleinere n zeigen noch rudimentäre Bildinformationen
 surface.setSize(dimX,dimY);  
 font = loadFont("ArialMT-18.vlw");
 background(255); 
 fill(0);noStroke();
 textFont(font);
}
/////////////////////////////////////////////////////
void draw(){
 if(!fertig){
     text("Zunächst mit Taste 's'einen Schlüssel erstellen",50,100);
     text("Das zu verschlüsselnde Bild mit Namen  bild.png muss im data-Ordner liegen.",50,150);
     text("Verschlüsseln mit 'v'-Taste!",50,180);
     text("Das zu entschlüsselnde Bild mit Namen  bildVerschluesselt.png muss im data-Ordner liegen.",50,210);
     text("Verschlüsseln mit 'e'-Taste!",50,240);
     text("Alle Schlüssel und alle Bild-Ergebnisse werden automatisch abgespeichert.",50,300);
 } 
 if(fertig){
   image(bild,0,0);
 }
}